@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-app.c 5102 2006-08-16 17:45:13Z pborelli $
+ * $Id: gedit-app.c 6128 2008-01-26 14:02:32Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,6 +44,10 @@
 #include "gedit-utils.h"
 #include "gedit-enum-types.h"
 
+
+#define GEDIT_PAGE_SETUP_FILE		"gedit-page-setup"
+#define GEDIT_PRINT_SETTINGS_FILE	"gedit-print-settings"
+
 #define GEDIT_APP_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), GEDIT_TYPE_APP, GeditAppPrivate))
 
 /* Properties */
@@ -57,7 +61,11 @@ struct _GeditAppPrivate
 {
 	GList	          *windows;
 	GeditWindow       *active_window;
+
 	GeditLockdownMask  lockdown;
+
+	GtkPageSetup      *page_setup;
+	GtkPrintSettings  *print_settings;
 };
 
 G_DEFINE_TYPE(GeditApp, gedit_app, G_TYPE_OBJECT)
@@ -69,6 +77,11 @@ gedit_app_finalize (GObject *object)
 
 	g_list_free (app->priv->windows);
 
+	if (app->priv->page_setup)
+		g_object_unref (app->priv->page_setup);
+	if (app->priv->print_settings)
+		g_object_unref (app->priv->print_settings);
+
 	G_OBJECT_CLASS (gedit_app_parent_class)->finalize (object);
 }
 
@@ -102,20 +115,214 @@ gedit_app_class_init (GeditAppClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_LOCKDOWN,
 					 g_param_spec_flags ("lockdown",
-					 		     "Lockdown",
-					 		     "The lockdown mask",
-					 		     GEDIT_TYPE_LOCKDOWN_MASK,
-					 		     0,
-					 		     G_PARAM_READABLE));
+							     "Lockdown",
+							     "The lockdown mask",
+							     GEDIT_TYPE_LOCKDOWN_MASK,
+							     0,
+							     G_PARAM_READABLE |
+							     G_PARAM_STATIC_STRINGS));
 
 	g_type_class_add_private (object_class, sizeof(GeditAppPrivate));
 }
 
+static gchar *
+get_accel_file (void)
+{
+	const gchar *home;
+
+	home = g_get_home_dir();
+
+	if (home != NULL)
+	{
+		return g_build_filename (home,
+					 ".gnome2",
+					 "accels"
+					 "gedit",
+					 NULL);
+	}
+
+	return NULL;
+}
+
+static void
+load_accels (void)
+{
+	gchar *filename;
+
+	filename = get_accel_file ();
+	if (filename != NULL)
+	{
+		gedit_debug_message (DEBUG_APP, "Loading keybindings from %s\n", filename);		
+		gtk_accel_map_load (filename);
+		g_free (filename);
+	}
+}
+
+static void
+save_accels (void)
+{
+	gchar *filename;
+
+	filename = get_accel_file ();
+	if (filename != NULL)
+	{
+		gedit_debug_message (DEBUG_APP, "Saving keybindings in %s\n", filename);		
+		gtk_accel_map_save (filename);
+		g_free (filename);
+	}
+}
+
+static gchar *
+get_page_setup_file (void)
+{
+	const gchar *home;
+
+	home = g_get_home_dir ();
+	if (home != NULL)
+	{
+		return g_build_filename (home,
+					 ".gnome2",
+					 "gedit",
+					 GEDIT_PAGE_SETUP_FILE,
+					 NULL);
+	}
+
+	return NULL;
+}
+
+static void
+load_page_setup (GeditApp *app)
+{
+	gchar *filename;
+	GError *error = NULL;
+
+	g_return_if_fail (app->priv->page_setup == NULL);
+
+	filename = get_page_setup_file ();
+
+	app->priv->page_setup = gtk_page_setup_new_from_file (filename,
+							      &error);
+	if (error)
+	{
+		/* Ignore file not found error */
+		if (error->domain != G_FILE_ERROR ||
+		    error->code != G_FILE_ERROR_NOENT)
+		{
+			g_warning (error->message);
+		}
+
+		g_error_free (error);
+	}
+
+	g_free (filename);
+
+	/* fall back to default settings */
+	if (app->priv->page_setup == NULL)
+		app->priv->page_setup = gtk_page_setup_new ();
+}
+
+static void
+save_page_setup (GeditApp *app)
+{
+	gchar *filename;
+	GError *error = NULL;
+
+	if (app->priv->page_setup == NULL)
+		return;
+
+	filename = get_page_setup_file ();
+
+	gtk_page_setup_to_file (app->priv->page_setup,
+				filename,
+				&error);
+	if (error)
+	{
+		g_warning (error->message);
+		g_error_free (error);
+	}
+
+	g_free (filename);
+}
+
+static gchar *
+get_print_settings_file (void)
+{
+	const gchar *home;
+
+	home = g_get_home_dir ();
+	if (home != NULL)
+	{
+		return g_build_filename (home,
+					 ".gnome2",
+					 "gedit",
+					 GEDIT_PRINT_SETTINGS_FILE,
+					 NULL);
+	}
+
+	return NULL;
+}
+
+static void
+load_print_settings (GeditApp *app)
+{
+	gchar *filename;
+	GError *error = NULL;
+
+	g_return_if_fail (app->priv->print_settings == NULL);
+
+	filename = get_print_settings_file ();
+
+	app->priv->print_settings = gtk_print_settings_new_from_file (filename,
+								      &error);
+	if (error)
+	{
+		/* Ignore file not found error */
+		if (error->domain != G_FILE_ERROR ||
+		    error->code != G_FILE_ERROR_NOENT)
+		{
+			g_warning (error->message);
+		}
+
+		g_error_free (error);
+	}
+
+	g_free (filename);
+
+	/* fall back to default settings */
+	if (app->priv->print_settings == NULL)
+		app->priv->print_settings = gtk_print_settings_new ();
+}
+
+static void
+save_print_settings (GeditApp *app)
+{
+	gchar *filename;
+	GError *error = NULL;
+
+	if (app->priv->print_settings == NULL)
+		return;
+
+	filename = get_print_settings_file ();
+
+	gtk_print_settings_to_file (app->priv->print_settings,
+				    filename,
+				    &error);
+	if (error)
+	{
+		g_warning (error->message);
+		g_error_free (error);
+	}
+
+	g_free (filename);
+}
+
 static void
 gedit_app_init (GeditApp *app)
 {
 	app->priv = GEDIT_APP_GET_PRIVATE (app);
-		
+
+	load_accels ();
+
 	/* initial lockdown state */
 	app->priv->lockdown = gedit_prefs_manager_get_lockdown ();
 }
@@ -213,6 +420,13 @@ window_destroy (GeditWindow *window,
 */					      
 	if (app->priv->windows == NULL)
 	{
+		/* Last window is gone... save some settings and exit */
+
+		save_accels ();
+
+		save_page_setup (app);
+		save_print_settings (app);
+
 		g_object_unref (app);
 	}
 }
@@ -399,9 +613,11 @@ gedit_app_get_active_window (GeditApp *app)
 }
 
 static gboolean
-is_in_workspace (GeditWindow *window,
-		 GdkScreen   *screen,
-		 gint         workspace)
+is_in_viewport (GeditWindow  *window,
+		GdkScreen    *screen,
+		gint          workspace,
+		gint          viewport_x,
+		gint          viewport_y)
 {
 	GdkScreen *s;
 	GdkDisplay *display;
@@ -410,7 +626,11 @@ is_in_workspace (GeditWindow *window,
 	gint cur_n;
 	gint n;
 	gint ws;
+	gint sc_width, sc_height;
+	gint x, y, width, height;
+	gint vp_x, vp_y;
 
+	/* Check for screen and display match */
 	display = gdk_screen_get_display (screen);
 	cur_name = gdk_display_get_name (display);
 	cur_n = gdk_screen_get_number (screen);
@@ -420,16 +640,50 @@ is_in_workspace (GeditWindow *window,
 	name = gdk_display_get_name (display);
 	n = gdk_screen_get_number (s);
 
-	ws = gedit_utils_get_window_workspace (GTK_WINDOW (window));
+	if (strcmp (cur_name, name) != 0 || cur_n != n)
+		return FALSE;
 
-	return ((strcmp (cur_name, name) == 0) && cur_n == n &&
-	        (ws == workspace || ws == GEDIT_ALL_WORKSPACES));
+	/* Check for workspace match */
+	ws = gedit_utils_get_window_workspace (GTK_WINDOW (window));
+	if (ws != workspace && ws != GEDIT_ALL_WORKSPACES)
+		return FALSE;
+
+	/* Check for viewport match */
+	gdk_window_get_position (GTK_WIDGET (window)->window, &x, &y);
+	gdk_drawable_get_size (GTK_WIDGET (window)->window, &width, &height);
+	gedit_utils_get_current_viewport (screen, &vp_x, &vp_y);
+	x += vp_x;
+	y += vp_y;
+
+	sc_width = gdk_screen_get_width (screen);
+	sc_height = gdk_screen_get_height (screen);
+
+	return x + width * .25 >= viewport_x &&
+	       x + width * .75 < viewport_x + sc_width &&
+	       y  >= viewport_y &&
+	       y + height < viewport_y + sc_height;
 }
 
+/**
+ * _gedit_app_get_window_in_viewport
+ * @app: the #GeditApp
+ * @screen: the #GdkScreen
+ * @workspace: the workspace number
+ * @viewport_x: the viewport horizontal origin
+ * @viewport_y: the viewport vertical origin
+ *
+ * Since a workspace can be larger than the screen, it is divided into several
+ * equal parts called viewports. This function retrives the #GeditWindow in
+ * the given viewport of the given workspace.
+ *
+ * Return value: the #GeditWindow in the given viewport of the given workspace.
+ */
 GeditWindow *
-_gedit_app_get_window_in_workspace (GeditApp  *app,
-				    GdkScreen *screen,
-				    gint       workspace)
+_gedit_app_get_window_in_viewport (GeditApp  *app,
+				   GdkScreen *screen,
+				   gint       workspace,
+				   gint       viewport_x,
+				   gint       viewport_y)
 {
 	GeditWindow *window;
 
@@ -442,7 +696,7 @@ _gedit_app_get_window_in_workspace (GeditApp  *app,
 
 	g_return_val_if_fail (GEDIT_IS_WINDOW (window), NULL);
 
-	if (is_in_workspace (window, screen, workspace))
+	if (is_in_viewport (window, screen, workspace, viewport_x, viewport_y))
 		return window;
 
 	/* otherwise try to see if there is a window on this workspace */
@@ -450,7 +704,7 @@ _gedit_app_get_window_in_workspace (GeditApp  *app,
 	{
 		window = l->data;
 
-		if (is_in_workspace (window, screen, workspace))
+		if (is_in_viewport (window, screen, workspace, viewport_x, viewport_y))
 			return window;
 	}
 
@@ -569,3 +823,54 @@ _gedit_app_set_lockdown_bit (GeditApp          *app,
 
 	app_lockdown_changed (app);
 }
+
+/* Returns a copy */
+GtkPageSetup *
+_gedit_app_get_default_page_setup (GeditApp *app)
+{
+	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
+
+	if (app->priv->page_setup == NULL)
+		load_page_setup (app);
+
+	return gtk_page_setup_copy (app->priv->page_setup);
+}
+
+void
+_gedit_app_set_default_page_setup (GeditApp     *app,
+				   GtkPageSetup *page_setup)
+{
+	g_return_if_fail (GEDIT_IS_APP (app));
+	g_return_if_fail (GTK_IS_PAGE_SETUP (page_setup));
+
+	if (app->priv->page_setup != NULL)
+		g_object_unref (app->priv->page_setup);
+
+	app->priv->page_setup = g_object_ref (page_setup);
+}
+
+/* Returns a copy */
+GtkPrintSettings *
+_gedit_app_get_default_print_settings (GeditApp *app)
+{
+	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
+
+	if (app->priv->print_settings == NULL)
+		load_print_settings (app);
+
+	return gtk_print_settings_copy (app->priv->print_settings);
+}
+
+void
+_gedit_app_set_default_print_settings (GeditApp         *app,
+				       GtkPrintSettings *settings)
+{
+	g_return_if_fail (GEDIT_IS_APP (app));
+	g_return_if_fail (GTK_IS_PRINT_SETTINGS (settings));
+
+	if (app->priv->print_settings != NULL)
+		g_object_unref (app->priv->print_settings);
+
+	app->priv->print_settings = g_object_ref (settings);
+}
+
