@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document.h,v 1.46 2006/02/19 18:01:09 pborelli Exp $
+ * $Id: gedit-document.h 6111 2008-01-23 08:26:18Z sfre $
  */
  
 #ifndef __GEDIT_DOCUMENT_H__
@@ -46,7 +46,6 @@ G_BEGIN_DECLS
  */
 #define GEDIT_TYPE_DOCUMENT              (gedit_document_get_type())
 #define GEDIT_DOCUMENT(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), GEDIT_TYPE_DOCUMENT, GeditDocument))
-#define GEDIT_DOCUMENT_CONST(obj)        (G_TYPE_CHECK_INSTANCE_CAST((obj), GEDIT_TYPE_DOCUMENT, GeditDocument const))
 #define GEDIT_DOCUMENT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST((klass), GEDIT_TYPE_DOCUMENT, GeditDocumentClass))
 #define GEDIT_IS_DOCUMENT(obj)           (G_TYPE_CHECK_INSTANCE_TYPE((obj), GEDIT_TYPE_DOCUMENT))
 #define GEDIT_IS_DOCUMENT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GEDIT_TYPE_DOCUMENT))
@@ -60,6 +59,19 @@ typedef enum
 
 } GeditSearchFlags;
 
+/**
+ * GeditDocumentSaveFlags:
+ * @GEDIT_DOCUMENT_SAVE_IGNORE_MTIME: save file despite external modifications.
+ * @GEDIT_DOCUMENT_SAVE_IGNORE_BACKUP: write the file directly without attempting to backup.
+ * @GEDIT_DOCUMENT_SAVE_PRESERVE_BACKUP: preserve previous backup file, needed to support autosaving.
+ */
+typedef enum
+{
+	GEDIT_DOCUMENT_SAVE_IGNORE_MTIME 	= 1 << 0,
+	GEDIT_DOCUMENT_SAVE_IGNORE_BACKUP	= 1 << 1,
+	GEDIT_DOCUMENT_SAVE_PRESERVE_BACKUP	= 1 << 2
+} GeditDocumentSaveFlags;
+
 /* Private structure type */
 typedef struct _GeditDocumentPrivate    GeditDocumentPrivate;
 
@@ -90,6 +102,12 @@ struct _GeditDocumentClass
 	void (* cursor_moved)		(GeditDocument    *document);
 
 	/* Document load */
+	void (* load)			(GeditDocument       *document,
+					 const gchar         *uri,
+					 const GeditEncoding *encoding,
+					 gint                 line_pos,
+					 gboolean             create);
+
 	void (* loading)		(GeditDocument    *document,
 					 GnomeVFSFileSize  size,
 					 GnomeVFSFileSize  total_size);
@@ -98,6 +116,11 @@ struct _GeditDocumentClass
 					 const GError     *error);
 
 	/* Document save */
+	void (* save)			(GeditDocument          *document,
+					 const gchar            *uri,
+					 const GeditEncoding    *encoding,
+					 GeditDocumentSaveFlags  flags);
+
 	void (* saving)			(GeditDocument    *document,
 					 GnomeVFSFileSize  size,
 					 GnomeVFSFileSize  total_size);
@@ -112,19 +135,6 @@ struct _GeditDocumentClass
 };
 
 
-typedef enum
-{
-	/* save file despite external modifications */
-	GEDIT_DOCUMENT_SAVE_IGNORE_MTIME 	= 1 << 0,
-
-	/* write the file directly without attempting to backup */
-	GEDIT_DOCUMENT_SAVE_IGNORE_BACKUP	= 1 << 1,
-	
-	/* preserve previous backup file, needed to support autosaving */
-	GEDIT_DOCUMENT_SAVE_PRESERVE_BACKUP	= 1 << 2
-} GeditDocumentSaveFlags;
-
-
 #define GEDIT_DOCUMENT_ERROR gedit_document_error_quark ()
 
 enum
@@ -178,11 +188,9 @@ void		 gedit_document_save_as 	(GeditDocument       *doc,
 gboolean	 gedit_document_is_untouched 	(GeditDocument       *doc);
 gboolean	 gedit_document_is_untitled 	(GeditDocument       *doc);
 
+gboolean	 gedit_document_is_local	(GeditDocument       *doc);
+
 gboolean	 gedit_document_get_deleted	(GeditDocument       *doc);
-/* Ancora da discutere
-gboolean	 gedit_document_get_externally_modified
-						(GeditDocument       *doc);
-*/
 
 gboolean	 gedit_document_goto_line 	(GeditDocument       *doc, 
 						 gint                 line);
@@ -232,14 +240,11 @@ gboolean	 gedit_document_get_enable_search_highlighting
 /* 
  * Non exported functions
  */
-gboolean	_gedit_document_is_saving_as	(GeditDocument       *doc);
-
-// CHECK: va bene un gint?
 glong		 _gedit_document_get_seconds_since_last_save_or_load 
 						(GeditDocument       *doc);
 
-/* private because the property will move to gtk */
-gboolean	 _gedit_document_get_has_selection
+/* Note: this is a sync stat: use only on local files */
+gboolean	_gedit_document_check_externally_modified
 						(GeditDocument       *doc);
 
 void		_gedit_document_search_region   (GeditDocument       *doc,
