@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-panel.c,v 1.7 2006/04/25 17:47:53 sfre Exp $
+ * $Id: gedit-panel.c 6112 2008-01-23 08:26:24Z sfre $
  */
 
 #include "gedit-panel.h"
@@ -52,8 +52,6 @@ struct _GeditPanelPrivate
 
 	/* Notebook */
 	GtkWidget *notebook;
-
-	GtkTooltips *tooltips;
 };
 
 typedef struct _GeditPanelItem GeditPanelItem;
@@ -100,8 +98,6 @@ G_DEFINE_TYPE(GeditPanel, gedit_panel, GTK_TYPE_VBOX)
 static void
 gedit_panel_finalize (GObject *obj)
 {
-	g_object_unref (GEDIT_PANEL (obj)->priv->tooltips);
-
 	if (G_OBJECT_CLASS (gedit_panel_parent_class)->finalize)
 		(*G_OBJECT_CLASS (gedit_panel_parent_class)->finalize) (obj);
 }
@@ -203,7 +199,10 @@ gedit_panel_class_init (GeditPanelClass *klass)
 							    "The panel's orientation",
 							    GTK_TYPE_ORIENTATION,
 							    GTK_ORIENTATION_VERTICAL,
-							    G_PARAM_WRITABLE | G_PARAM_READABLE | G_PARAM_CONSTRUCT_ONLY));
+							    G_PARAM_WRITABLE |
+							    G_PARAM_READABLE |
+							    G_PARAM_CONSTRUCT_ONLY |
+							    G_PARAM_STATIC_STRINGS));
 
 	widget_class->grab_focus = gedit_panel_grab_focus;
 
@@ -416,27 +415,47 @@ gedit_panel_init (GeditPanel *panel)
 {
 	panel->priv = GEDIT_PANEL_GET_PRIVATE (panel);
 	g_return_if_fail (panel->priv != NULL);	
-	
-	panel->priv->tooltips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (panel->priv->tooltips));
-	gtk_object_sink (GTK_OBJECT (panel->priv->tooltips));
+}
+
+static void
+button_style_set_cb (GtkWidget *button,
+		     GtkStyle *previous_style,
+		     gpointer user_data)
+{
+	gint h, w;
+
+	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (button),
+					   GTK_ICON_SIZE_MENU, &w, &h);
+
+	gtk_widget_set_size_request (button, w + 2, h + 2);
 }
 
 static GtkWidget *
 create_small_button (GtkWidget *image)
 {
-	/* This is quite ugly and sometimes don't give good results */
-	gint w, h;
-	GtkWidget *button = gtk_button_new ();	
-	gtk_widget_show (image);		    
-	gtk_button_set_relief (GTK_BUTTON (button),
-			       GTK_RELIEF_NONE);
+	GtkWidget *button;
+	GtkRcStyle *rcstyle;
+	
+	button = gtk_button_new ();
+	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 
-	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
-	gtk_widget_set_size_request (image, w, h); 
-	gtk_widget_set_size_request (button, w + 4, h + 4);
-	gtk_container_add (GTK_CONTAINER (button), image);
+	/* don't allow focus on the close button */
 	gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
+
+	/* make it as small as possible */
+	rcstyle = gtk_rc_style_new ();
+	rcstyle->xthickness = rcstyle->ythickness = 0;
+	gtk_widget_modify_style (button, rcstyle);
+	gtk_rc_style_unref (rcstyle);
+
+	gtk_widget_show (image);
+
+	gtk_container_add (GTK_CONTAINER (button), image);
+
+	/* Set minimal size */
+	g_signal_connect (button, "style-set",
+			  G_CALLBACK (button_style_set_cb), NULL);
+
 	return button;
 }
 
@@ -486,7 +505,7 @@ build_horizontal_panel (GeditPanel *panel)
 			    FALSE, 
 			    0);
 
-	image = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
 	close_button = create_small_button (image);
 
 	gtk_box_pack_start (GTK_BOX (sidebar),
@@ -495,12 +514,9 @@ build_horizontal_panel (GeditPanel *panel)
 			    FALSE, 
 			    0);
 
-	gtk_tooltips_set_tip (panel->priv->tooltips,
-			      close_button,
-			      _("Hide panel"),
-			      NULL);
+	gtk_widget_set_tooltip_text (close_button, _("Hide panel"));
 
-	g_signal_connect (G_OBJECT (close_button),
+	g_signal_connect (close_button,
 			  "clicked",
                           G_CALLBACK (close_button_clicked_cb),
                           panel);
@@ -572,16 +588,13 @@ build_vertical_panel (GeditPanel *panel)
 			    FALSE, 
 			    0);
 
-	gtk_tooltips_set_tip (panel->priv->tooltips,
-			      close_button,
-			      _("Hide panel"),
-			      NULL);
+	gtk_widget_set_tooltip_text (close_button, _("Hide panel"));
 
-	g_signal_connect (G_OBJECT (close_button), 
+	g_signal_connect (close_button,
 			  "clicked",
                           G_CALLBACK (close_button_clicked_cb),
                           panel);
-                 
+
 	gtk_widget_show_all (title_hbox);
 
 	gtk_box_pack_start (GTK_BOX (panel),
@@ -658,10 +671,7 @@ build_tab_label (GeditPanel  *panel,
         gtk_misc_set_padding (GTK_MISC (label), 0, 0);
 	gtk_box_pack_start (GTK_BOX (label_hbox), label, TRUE, TRUE, 0);
 
-	gtk_tooltips_set_tip (panel->priv->tooltips,
-			      label_ebox,
-			      name,
-			      NULL);
+	gtk_widget_set_tooltip_text (label_ebox, name);
 
 	gtk_widget_show_all (hbox);
 
@@ -775,10 +785,6 @@ gedit_panel_remove_item (GeditPanel *panel,
 		           NULL);
 
 	ebox = g_object_get_data (G_OBJECT (item), "label-ebox");
-	gtk_tooltips_set_tip (GTK_TOOLTIPS (panel->priv->tooltips), 
-			      ebox, 
-			      NULL, 
-			      NULL);  
 	
 	/* ref the item to keep it alive during signal emission */
 	g_object_ref (G_OBJECT (item));
@@ -843,11 +849,83 @@ gedit_panel_item_is_active (GeditPanel *panel,
 GtkOrientation
 gedit_panel_get_orientation (GeditPanel *panel)
 {
+	g_return_val_if_fail (GEDIT_IS_PANEL (panel), GTK_ORIENTATION_VERTICAL);
+
 	return panel->priv->orientation;
 }
 
 gint
 gedit_panel_get_n_items (GeditPanel *panel)
 {
+	g_return_val_if_fail (GEDIT_IS_PANEL (panel), -1);
+
 	return gtk_notebook_get_n_pages (GTK_NOTEBOOK (panel->priv->notebook));
 }
+
+gint
+_gedit_panel_get_active_item_id (GeditPanel *panel)
+{
+	gint cur_page;
+	GtkWidget *item;
+	GeditPanelItem *data;
+
+	g_return_val_if_fail (GEDIT_IS_PANEL (panel), 0);
+
+	cur_page = gtk_notebook_get_current_page (
+				GTK_NOTEBOOK (panel->priv->notebook));
+	if (cur_page == -1)
+		return 0;
+
+	item = gtk_notebook_get_nth_page (
+				GTK_NOTEBOOK (panel->priv->notebook),
+				cur_page);
+
+	/* FIXME: for now we use as the hash of the name as id.
+	 * However the name is not guaranteed to be unique and
+	 * it is a translated string, so it's subotimal, but should
+	 * be good enough for now since we don't want to add an
+	 * ad hoc id argument.
+	 */
+
+	data = (GeditPanelItem *)g_object_get_data (G_OBJECT (item),
+					            PANEL_ITEM_KEY);
+	g_return_val_if_fail (data != NULL, 0);
+
+	return g_str_hash (data->name);
+}
+
+void
+_gedit_panel_set_active_item_by_id (GeditPanel *panel,
+				    gint        id)
+{
+	gint n, i;
+
+	g_return_if_fail (GEDIT_IS_PANEL (panel));
+
+	if (id == 0)
+		return;
+
+	n = gtk_notebook_get_n_pages (
+				GTK_NOTEBOOK (panel->priv->notebook));
+
+	for (i = 0; i < n; i++)
+	{
+		GtkWidget *item;
+		GeditPanelItem *data;
+
+		item = gtk_notebook_get_nth_page (
+				GTK_NOTEBOOK (panel->priv->notebook), i);
+
+		data = (GeditPanelItem *)g_object_get_data (G_OBJECT (item),
+						            PANEL_ITEM_KEY);
+		g_return_if_fail (data != NULL);
+
+		if (g_str_hash (data->name) == id)
+		{
+			gtk_notebook_set_current_page (
+				GTK_NOTEBOOK (panel->priv->notebook), i);
+
+			return;
+		}
+	}
+}
