@@ -26,7 +26,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-prefs-manager-app.c 5737 2007-07-23 16:32:51Z pborelli $
+ * $Id: gedit-prefs-manager-app.c 5854 2007-08-27 15:38:06Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,7 +43,7 @@
 #include "gedit-view.h"
 #include "gedit-window.h"
 #include "gedit-window-private.h"
-#include "gedit-source-style-manager.h"
+#include "gedit-style-scheme-manager.h"
 
 static void gedit_prefs_manager_editor_font_changed	(GConfClient *client,
 							 guint        cnxn_id,
@@ -197,6 +197,8 @@ get_gedit_state_file ()
 				g_warning ("Could not load gedit state file: %s\n",
 					   err->message);
 			}
+
+			g_error_free (err);
 		}
 
 		g_free (path);
@@ -237,6 +239,8 @@ gedit_state_get_int (const gchar *group,
 
 		if (defval > 0)
 			*result = defval;
+
+		g_error_free (err);
 	}
 	else
 	{
@@ -308,6 +312,9 @@ gedit_state_file_sync ()
 	ret = TRUE;
 
  out:
+	if (err != NULL)
+		g_error_free (err);
+
 	g_free (content);
 	g_free (path);
 
@@ -762,7 +769,7 @@ gedit_prefs_manager_editor_font_changed (GConfClient *client,
 	{
 		/* Note: we use def=FALSE to avoid GeditView to query gconf */
 		gedit_view_set_font (GEDIT_VIEW (l->data), FALSE,  font);
-		gtk_source_view_set_tabs_width (GTK_SOURCE_VIEW (l->data), ts);
+		gtk_source_view_set_tab_width (GTK_SOURCE_VIEW (l->data), ts);
 
 		l = l->next;
 	}
@@ -808,7 +815,7 @@ gedit_prefs_manager_system_font_changed (GConfClient *client,
 		/* Note: we use def=FALSE to avoid GeditView to query gconf */
 		gedit_view_set_font (GEDIT_VIEW (l->data), FALSE, font);
 
-		gtk_source_view_set_tabs_width (GTK_SOURCE_VIEW (l->data), ts);
+		gtk_source_view_set_tab_width (GTK_SOURCE_VIEW (l->data), ts);
 		l = l->next;
 	}
 
@@ -829,24 +836,24 @@ gedit_prefs_manager_tabs_size_changed (GConfClient *client,
 
 	if (strcmp (entry->key, GPM_TABS_SIZE) == 0)
 	{
-		gint tabs_size;
+		gint tab_width;
 		GList *views;
 		GList *l;
-		
+
 		if (entry->value->type == GCONF_VALUE_INT)
-			tabs_size = gconf_value_get_int (entry->value);
+			tab_width = gconf_value_get_int (entry->value);
 		else
-			tabs_size = GPM_DEFAULT_TABS_SIZE;
+			tab_width = GPM_DEFAULT_TABS_SIZE;
 	
-		tabs_size = CLAMP (tabs_size, 1, 24);
+		tab_width = CLAMP (tab_width, 1, 24);
 
 		views = gedit_app_get_views (gedit_app_get_default ());
 		l = views;
 
 		while (l != NULL)
 		{
-			gtk_source_view_set_tabs_width (GTK_SOURCE_VIEW (l->data), 
-							tabs_size);
+			gtk_source_view_set_tab_width (GTK_SOURCE_VIEW (l->data), 
+						       tab_width);
 
 			l = l->next;
 		}
@@ -858,7 +865,7 @@ gedit_prefs_manager_tabs_size_changed (GConfClient *client,
 		gboolean enable;
 		GList *views;
 		GList *l;
-		
+
 		if (entry->value->type == GCONF_VALUE_BOOL)
 			enable = gconf_value_get_bool (entry->value);	
 		else
@@ -1039,8 +1046,8 @@ gedit_prefs_manager_bracket_matching_changed (GConfClient *client,
 
 		while (l != NULL)
 		{
-			gtk_source_buffer_set_check_brackets (GTK_SOURCE_BUFFER (l->data),
-							      enable);
+			gtk_source_buffer_set_highlight_matching_brackets (GTK_SOURCE_BUFFER (l->data),
+									   enable);
 
 			l = l->next;
 		}
@@ -1154,8 +1161,8 @@ gedit_prefs_manager_right_margin_changed (GConfClient *client,
 
 		while (l != NULL)
 		{
-			gtk_source_view_set_margin (GTK_SOURCE_VIEW (l->data),
-						    pos);
+			gtk_source_view_set_right_margin_position (GTK_SOURCE_VIEW (l->data),
+								   pos);
 
 			l = l->next;
 		}
@@ -1178,8 +1185,8 @@ gedit_prefs_manager_right_margin_changed (GConfClient *client,
 
 		while (l != NULL)
 		{
-			gtk_source_view_set_show_margin (GTK_SOURCE_VIEW (l->data),
-							 display);
+			gtk_source_view_set_show_right_margin (GTK_SOURCE_VIEW (l->data),
+							       display);
 
 			l = l->next;
 		}
@@ -1275,8 +1282,8 @@ gedit_prefs_manager_syntax_hl_enable_changed (GConfClient *client,
 		{
 			g_return_if_fail (GTK_IS_SOURCE_BUFFER (l->data));
 
-			gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (l->data),
-							 enable);
+			gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (l->data),
+								enable);
 
 			l = l->next;
 		}
@@ -1354,8 +1361,8 @@ gedit_prefs_manager_source_style_scheme_changed (GConfClient *client,
 
 	if (strcmp (entry->key, GPM_SOURCE_STYLE_SCHEME) == 0)
 	{
+		static gchar *old_scheme = NULL;
 		const gchar *scheme;
-		gboolean changed;
 		GtkSourceStyleScheme *style;
 		GList *docs;
 		GList *l;
@@ -1365,14 +1372,30 @@ gedit_prefs_manager_source_style_scheme_changed (GConfClient *client,
 		else
 			scheme = GPM_DEFAULT_SOURCE_STYLE_SCHEME;
 
-		changed = _gedit_source_style_manager_set_default_scheme
-					(gedit_get_source_style_manager (),
-					 scheme);
-		if (!changed)
-			return;
+		if (old_scheme != NULL && (strcmp (scheme, old_scheme) == 0))
+		    	return;
 
-		style = gedit_source_style_manager_get_default_scheme
-					(gedit_get_source_style_manager ());
+		g_free (old_scheme);
+		old_scheme = g_strdup (scheme);
+
+		style = gtk_source_style_scheme_manager_get_scheme (
+				gedit_get_style_scheme_manager (),
+				scheme);
+
+		if (style == NULL)
+		{
+			g_warning ("Default style scheme '%s' not found, falling back to 'classic'", scheme);
+			
+			style = gtk_source_style_scheme_manager_get_scheme (
+				gedit_get_style_scheme_manager (),
+				"classic");
+
+			if (style == NULL) 
+			{
+				g_warning ("Style scheme 'classic' cannot be found, check your GtkSourceView installation.");
+				return;
+			}
+		}
 
 		docs = gedit_app_get_documents (gedit_app_get_default ());
 		for (l = docs; l != NULL; l = l->next)
