@@ -222,8 +222,10 @@ gedit_python_module_class_init (GeditPythonModuleClass *class)
 					      "Module Name",
 					      "The Python module to load for this plugin",
 					      NULL,
-					      G_PARAM_WRITABLE | G_PARAM_READABLE | G_PARAM_CONSTRUCT_ONLY));
-					      
+					      G_PARAM_READWRITE |
+					      G_PARAM_CONSTRUCT_ONLY |
+					      G_PARAM_STATIC_STRINGS));
+
 	g_object_class_install_property
 			(object_class,
 			 PROP_PATH,
@@ -231,7 +233,9 @@ gedit_python_module_class_init (GeditPythonModuleClass *class)
 					      "Path",
 					      "The Python path to use when loading this module",
 					      NULL,
-					      G_PARAM_WRITABLE | G_PARAM_READABLE | G_PARAM_CONSTRUCT_ONLY));
+					      G_PARAM_READWRITE |
+					      G_PARAM_CONSTRUCT_ONLY |
+					      G_PARAM_STATIC_STRINGS));
 
 	g_type_class_add_private (object_class, sizeof (GeditPythonModulePrivate));
 	
@@ -389,8 +393,8 @@ gedit_init_pygtksourceview (void)
 gboolean
 gedit_python_init (void)
 {
-	PyObject *mdict, *path, *tuple;
-	PyObject *sys_path, *gedit, *geditutils, *geditcommands;
+	PyObject *mdict, *tuple;
+	PyObject *gedit, *geditutils, *geditcommands;
 	PyObject *gettext, *install, *gettext_args;
 	struct sigaction old_sigint;
 	gint res;
@@ -484,12 +488,6 @@ gedit_python_init (void)
 		goto python_init_error;
 	}	
 	
-	/* sys.path.insert(0, ...) for system-wide plugins */
-	sys_path = PySys_GetObject ("path");
-	path = PyString_FromString (GEDIT_PLUGINDIR "/");
-	PyList_Insert (sys_path, 0, path);
-	Py_DECREF(path);
-
 	/* import gedit */
 	gedit = Py_InitModule ("gedit", pygedit_functions);
 	mdict = PyModule_GetDict (gedit);
