@@ -43,6 +43,7 @@
 #define NAUTILUS_CLICK_POLICY_KEY	"click_policy"
 #define NAUTILUS_ENABLE_DELETE_KEY	"enable_delete"
 #define NAUTILUS_CONFIRM_TRASH_KEY	"confirm_trash"
+#define TERMINAL_EXEC_KEY		"/desktop/gnome/applications/terminal/exec"
 
 #define GEDIT_FILE_BROWSER_PLUGIN_GET_PRIVATE(object)	(G_TYPE_INSTANCE_GET_PRIVATE ((object), GEDIT_TYPE_FILE_BROWSER_PLUGIN, GeditFileBrowserPluginPrivate))
 
@@ -466,14 +467,20 @@ get_terminal (void)
 	gchar * terminal;
 
 	client = gconf_client_get_default ();
-	terminal = gconf_client_get_string (client, "/desktop/gnome/applications/terminal/exec", NULL);
-	
-	if (terminal == NULL)
-		terminal = g_strdup(getenv("TERM"));
-	
-	if (terminal == NULL)
-		terminal = g_strdup("xterm");
-	
+	terminal = gconf_client_get_string (client,
+					    TERMINAL_EXEC_KEY,
+					    NULL);
+	g_object_unref (client);
+
+	if (terminal == NULL) {
+		const gchar *term = g_getenv ("TERM");
+
+		if (term != NULL)
+			terminal = g_strdup (term);
+		else
+			terminal = g_strdup ("xterm");
+	}
+
 	return terminal;
 }
 
@@ -612,17 +619,19 @@ add_popup_ui (GeditWindow * window)
 	data = get_plugin_data (window);
 	manager = gedit_file_browser_widget_get_ui_manager (data->tree_widget);
 
-	action_group =
-	    gtk_action_group_new ("FileBrowserPluginExtra");
-	gtk_action_group_add_actions (action_group, extra_actions,
+	action_group = gtk_action_group_new ("FileBrowserPluginExtra");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group,
+				      extra_actions,
 				      G_N_ELEMENTS (extra_actions),
 				      window);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	data->action_group = action_group;
-	
-	action_group =
-	    gtk_action_group_new ("FileBrowserPluginSingleSelectionExtra");
-	gtk_action_group_add_actions (action_group, extra_single_selection_actions,
+
+	action_group = gtk_action_group_new ("FileBrowserPluginSingleSelectionExtra");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group,
+				      extra_single_selection_actions,
 				      G_N_ELEMENTS (extra_single_selection_actions),
 				      window);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
@@ -1044,13 +1053,11 @@ on_tab_added_cb (GeditWindow * window,
                  GeditTab * tab,
                  GeditFileBrowserPluginData * data)
 {
-	GConfClient * client;
-	GeditDocument * doc;
+	GConfClient *client;
 	gboolean open;
-	gchar *uri;
 
 	client = gconf_client_get_default ();
-	
+
 	if (!client)
 		return;
 
@@ -1059,26 +1066,30 @@ on_tab_added_cb (GeditWindow * window,
 	                              NULL);
 
 	if (open) {
+		GeditDocument *doc;
+
 		doc = gedit_tab_get_document (tab);
-		
+
 		if (!gedit_document_is_untitled (doc)) {
+			gchar *uri;
+
 			uri = gedit_document_get_uri (doc);
-			
+
 			if (gedit_utils_uri_has_file_scheme (uri)) {
 				prepare_auto_root (data);
 				set_root_from_doc (data, doc);
 			}
+
+			g_free (uri);
 		}
-		
 	}
-	
+
 	g_object_unref (client);
-	
+
 	/* Disconnect this signal, it's only called once */
 	g_signal_handlers_disconnect_by_func (window, 
 	                                      G_CALLBACK (on_tab_added_cb),
 	                                      data);
-	
 }
 
 static gchar *
@@ -1131,31 +1142,31 @@ on_confirm_no_trash_cb (GeditFileBrowserWidget * widget,
 }
 
 static gboolean
-on_confirm_delete_cb (GeditFileBrowserWidget * widget,
-                      GeditFileBrowserStore * store,
+on_confirm_delete_cb (GeditFileBrowserWidget *widget,
+                      GeditFileBrowserStore *store,
                       GList *paths,
-                      GeditWindow * window)
+                      GeditWindow *window)
 {
 	gchar *normal;
 	gchar *message;
 	gchar *secondary;
 	gboolean result;
 	GeditFileBrowserPluginData *data;
-	
+
 	data = get_plugin_data (window);
 
 	if (!data->confirm_trash)
 		return TRUE;
-	
+
 	if (paths->next == NULL) {
 		normal = get_filename_from_path (GTK_TREE_MODEL (store), (GtkTreePath *)(paths->data));
-		message = g_strdup_printf (_("Are you sure you want to\npermanently delete \"%s\"?"), normal);
+		message = g_strdup_printf (_("Are you sure you want to permanently delete \"%s\"?"), normal);
 		g_free (normal);
 	} else {
-		message = g_strdup (_("Are you sure you want to\npermanently delete the selected files?."));
+		message = g_strdup (_("Are you sure you want to permanently delete the selected files?"));
 	}
 
-	secondary = _("If you delete an item it is permanently lost.");
+	secondary = _("If you delete an item, it is permanently lost.");
 
 	result = gedit_file_browser_utils_confirmation_dialog (window,
 	                                                       GTK_MESSAGE_QUESTION, 
