@@ -25,7 +25,9 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <string.h>
 #include <glib/gi18n-lib.h>
+#include <gdk/gdkkeysyms.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomeui/libgnomeui.h>
 #include <gedit/gedit-utils.h>
@@ -68,7 +70,8 @@ enum
 {
 	PROP_0,
 	
-	PROP_FILTER_PATTERN
+	PROP_FILTER_PATTERN,
+	PROP_ENABLE_DELETE
 };
 
 /* Signals */
@@ -118,7 +121,6 @@ struct _GeditFileBrowserWidgetPrivate
 
 	GtkWidget *combo;
 	GtkTreeStore *combo_model;
-	GdkCursor *cursor;
 
 	GtkWidget *filter_expander;
 	GtkWidget *filter_entry;
@@ -126,6 +128,8 @@ struct _GeditFileBrowserWidgetPrivate
 	GtkUIManager *manager;
 	GtkActionGroup *action_group;
 	GtkActionGroup *action_group_selection;
+	GtkActionGroup *action_group_single_selection;
+	GtkActionGroup *action_group_single_most_selection;
 	GtkActionGroup *action_group_sensitive;
 
 	GSList *signal_pool;
@@ -142,8 +146,12 @@ struct _GeditFileBrowserWidgetPrivate
 	GtkWidget *location_previous_menu;
 	GtkWidget *location_next_menu;
 	GtkWidget *current_location_menu_item;
+	
+	gboolean enable_delete;
 };
 
+static void set_enable_delete		       (GeditFileBrowserWidget *obj,
+						gboolean enable);
 static void on_model_set                       (GObject * gobject, 
 						GParamSpec * arg1,
 						GeditFileBrowserWidget * obj);
@@ -298,9 +306,6 @@ gedit_file_browser_widget_finalize (GObject * object)
 	GList *loc;
 
 	remove_path_items (obj);
-
-	gdk_cursor_unref (obj->priv->cursor);
-
 	gedit_file_browser_store_set_filter_func (obj->priv->file_store,
 						  NULL, NULL);
 
@@ -338,6 +343,9 @@ gedit_file_browser_widget_get_property (GObject    *object,
 		case PROP_FILTER_PATTERN:
 			g_value_set_string (value, obj->priv->filter_pattern_str);
 			break;
+		case PROP_ENABLE_DELETE:
+			g_value_set_boolean (value, obj->priv->enable_delete);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -357,6 +365,10 @@ gedit_file_browser_widget_set_property (GObject      *object,
 		case PROP_FILTER_PATTERN:
 			gedit_file_browser_widget_set_filter_pattern (obj,
 			                                              g_value_get_string (value));
+			break;
+		case PROP_ENABLE_DELETE:
+			set_enable_delete (obj, g_value_get_boolean (value));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -379,6 +391,13 @@ gedit_file_browser_widget_class_init (GeditFileBrowserWidgetClass * klass)
 					 		      "The filter pattern",
 					 		      NULL,
 					 		      G_PARAM_READWRITE));
+	g_object_class_install_property (object_class, PROP_ENABLE_DELETE,
+					 g_param_spec_boolean ("enable-delete",
+					 		       "Enable delete",
+					 		       "Enable permanently deleting items",
+					 		       TRUE,
+					 		       G_PARAM_READWRITE |
+					 		       G_PARAM_CONSTRUCT));
 
 	signals[URI_ACTIVATED] =
 	    g_signal_new ("uri-activated",
@@ -403,11 +422,11 @@ gedit_file_browser_widget_class_init (GeditFileBrowserWidgetClass * klass)
 	                                   confirm_delete),
 	                  g_signal_accumulator_true_handled,
 	                  NULL,
-	                  gedit_file_browser_marshal_BOOL__OBJECT_BOXED,
+	                  gedit_file_browser_marshal_BOOL__OBJECT_POINTER,
 	                  G_TYPE_BOOLEAN,
 	                  2,
 	                  G_TYPE_OBJECT,
-	                  GTK_TYPE_TREE_ITER);
+	                  GTK_TYPE_POINTER);
 
 	signals[CONFIRM_NO_TRASH] =
 	    g_signal_new ("confirm-no-trash", G_OBJECT_CLASS_TYPE (object_class),
@@ -416,11 +435,11 @@ gedit_file_browser_widget_class_init (GeditFileBrowserWidgetClass * klass)
 	                                   confirm_no_trash),
 	                  g_signal_accumulator_true_handled,
 	                  NULL,
-	                  gedit_file_browser_marshal_BOOL__OBJECT_BOXED,
+	                  gedit_file_browser_marshal_BOOL__OBJECT_POINTER,
 	                  G_TYPE_BOOLEAN,
 	                  2,
 	                  G_TYPE_OBJECT,
-	                  GTK_TYPE_TREE_ITER);
+	                  GTK_TYPE_POINTER);
 
 	g_type_class_add_private (object_class,
 				  sizeof (GeditFileBrowserWidgetPrivate));
@@ -505,8 +524,9 @@ get_from_bookmark_uri (GeditFileBrowserWidget * obj, GnomeVFSURI * guri,
 		return FALSE;
 	
 	item = (NameIcon *)data;
-	*name = item->name;
-	*icon = item->icon;
+
+	*name = g_strdup (item->name);
+	*icon = g_object_ref (item->icon);
 
 	return TRUE;
 }
@@ -558,10 +578,8 @@ insert_path_item (GeditFileBrowserWidget * obj,
 
 		g_free (str);
 		g_free (str2);
-	} else {
-		unescape = g_strdup (unescape);
 	}
-	
+
 	gtk_tree_store_insert_after (obj->priv->combo_model, iter, NULL,
 				     after);
 	gtk_tree_store_set (obj->priv->combo_model, 
@@ -573,6 +591,9 @@ insert_path_item (GeditFileBrowserWidget * obj,
 			    COLUMN_ID, PATH_ID, 
 			    -1);
 
+	if (icon)
+		g_object_unref (icon);
+
 	g_free (unescape);
 }
 
@@ -707,21 +728,29 @@ fill_combo_model (GeditFileBrowserWidget * obj)
 {
 	GtkTreeStore *store = obj->priv->combo_model;
 	GtkTreeIter iter;
+	GdkPixbuf *icon;
+
+	icon = gedit_file_browser_utils_pixbuf_from_theme (GTK_STOCK_HOME, GTK_ICON_SIZE_MENU);
 
 	gtk_tree_store_append (store, &iter, NULL);
 	gtk_tree_store_set (store, &iter,
-			    COLUMN_ICON, gedit_file_browser_utils_pixbuf_from_theme (GTK_STOCK_HOME, GTK_ICON_SIZE_MENU),
+			    COLUMN_ICON, icon,
 			    COLUMN_NAME, _("Bookmarks"),
 			    COLUMN_ID, BOOKMARKS_ID, -1);
+	g_object_unref (icon);
+
 #if 0
+	icon = gedit_file_browser_utils_pixbuf_from_theme (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
+
 	gtk_tree_store_append (store, &iter, NULL);
 	gtk_tree_store_set (store, &iter,
-			    COLUMN_ICON, gedit_file_browser_utils_pixbuf_from_theme (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU),
+			    COLUMN_ICON, icon,
 			    COLUMN_NAME, _("Recent Files"),
 			    COLUMN_ID, RECENTS_ID, -1);
+	g_object_unref (icon);
 #endif
-	gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX
-					      (obj->priv->combo),
+
+	gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (obj->priv->combo),
 					      separator_func, obj, NULL);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (obj->priv->combo), 0);
 }
@@ -795,25 +824,11 @@ create_combo (GeditFileBrowserWidget * obj)
 
 static GtkActionEntry toplevel_actions[] = 
 {
-	{"FilterMenuAction", NULL, "_Filter"}
-};
-
-static const GtkActionEntry tree_actions[] = 
-{
-	{"DirectoryNew", GTK_STOCK_ADD, N_("_New Folder"), NULL,
-	 N_("Add new empty folder"),
-	 G_CALLBACK (on_action_directory_new)},
-	{"FileNew", GTK_STOCK_NEW, N_("New F_ile"), NULL,
-	 N_("Add new empty file"), G_CALLBACK (on_action_file_new)},
-	{"DirectoryUp", GTK_STOCK_GO_UP, N_("Up"), NULL,
-	 N_("Open the parent folder"), G_CALLBACK (on_action_directory_up)}
+	{"FilterMenuAction", NULL, N_("_Filter")}
 };
 
 static const GtkActionEntry tree_actions_selection[] = 
 {
-	{"FileRename", NULL, N_("_Rename"), NULL,
-	 N_("Rename selected file or folder"),
-	 G_CALLBACK (on_action_file_rename)},
 	{"FileMoveToTrash", "gnome-stock-trash", N_("_Move To Trash"), NULL,
 	 N_("Move selected file or folder to trash"),
 	 G_CALLBACK (on_action_file_move_to_trash)},
@@ -822,6 +837,28 @@ static const GtkActionEntry tree_actions_selection[] =
 	 G_CALLBACK (on_action_file_delete)}
 };
 
+static const GtkActionEntry tree_actions[] =
+{
+	{"DirectoryUp", GTK_STOCK_GO_UP, N_("Up"), NULL,
+	 N_("Open the parent folder"), G_CALLBACK (on_action_directory_up)}
+};
+
+static const GtkActionEntry tree_actions_single_most_selection[] =
+{
+	{"DirectoryNew", GTK_STOCK_ADD, N_("_New Folder"), NULL,
+	 N_("Add new empty folder"),
+	 G_CALLBACK (on_action_directory_new)},
+	{"FileNew", GTK_STOCK_NEW, N_("New F_ile"), NULL,
+	 N_("Add new empty file"), G_CALLBACK (on_action_file_new)}
+};
+
+static const GtkActionEntry tree_actions_single_selection[] = 
+{
+	{"FileRename", NULL, N_("_Rename"), NULL,
+	 N_("Rename selected file or folder"),
+	 G_CALLBACK (on_action_file_rename)}
+};
+
 static const GtkActionEntry tree_actions_sensitive[] = 
 {
 	{"DirectoryPrevious", GTK_STOCK_GO_BACK, N_("_Previous Location"),
@@ -870,46 +907,70 @@ create_toolbar (GeditFileBrowserWidget * obj)
 		return;
 	}
 
-	action_group =
-	    gtk_action_group_new ("FileBrowserWidgetActionGroupToplevel");
-	gtk_action_group_add_actions (action_group, toplevel_actions,
+	action_group = gtk_action_group_new ("FileBrowserWidgetActionGroupToplevel");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group,
+				      toplevel_actions,
 				      G_N_ELEMENTS (toplevel_actions),
 				      obj);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 
-	action_group =
-	    gtk_action_group_new ("FileBrowserWidgetActionGroup");
-	gtk_action_group_add_actions (action_group, tree_actions,
-				      G_N_ELEMENTS (tree_actions), obj);
+	action_group = gtk_action_group_new ("FileBrowserWidgetActionGroup");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group,
+				      tree_actions,
+				      G_N_ELEMENTS (tree_actions),
+				      obj);
 	gtk_action_group_add_toggle_actions (action_group,
 					     tree_actions_toggle,
-					     G_N_ELEMENTS
-					     (tree_actions_toggle), obj);
+					     G_N_ELEMENTS (tree_actions_toggle),
+					     obj);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	obj->priv->action_group = action_group;
 
-	action_group =
-	    gtk_action_group_new ("FileBrowserWidgetSelectionActionGroup");
-	gtk_action_group_add_actions (action_group, tree_actions_selection,
-				      G_N_ELEMENTS
-				      (tree_actions_selection), obj);
+	action_group = gtk_action_group_new ("FileBrowserWidgetSelectionActionGroup");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group,
+				      tree_actions_selection,
+				      G_N_ELEMENTS (tree_actions_selection),
+				      obj);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	obj->priv->action_group_selection = action_group;
 
-	action_group =
-	    gtk_action_group_new ("FileBrowserWidgetSensitiveActionGroup");
-	gtk_action_group_add_actions (action_group, tree_actions_sensitive,
-				      G_N_ELEMENTS
-				      (tree_actions_sensitive), obj);
+	action_group = gtk_action_group_new ("FileBrowserWidgetSingleSelectionActionGroup");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group,
+				      tree_actions_single_selection,
+				      G_N_ELEMENTS (tree_actions_single_selection),
+				      obj);
+	gtk_ui_manager_insert_action_group (manager, action_group, 0);
+	obj->priv->action_group_single_selection = action_group;
+
+	action_group = gtk_action_group_new ("FileBrowserWidgetSingleMostSelectionActionGroup");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group,
+				      tree_actions_single_most_selection,
+				      G_N_ELEMENTS (tree_actions_single_most_selection),
+				      obj);
+	gtk_ui_manager_insert_action_group (manager, action_group, 0);
+	obj->priv->action_group_single_most_selection = action_group;
+
+	action_group = gtk_action_group_new ("FileBrowserWidgetSensitiveActionGroup");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group,
+				      tree_actions_sensitive,
+				      G_N_ELEMENTS (tree_actions_sensitive),
+				      obj);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	obj->priv->action_group_sensitive = action_group;
 
-	gtk_action_set_sensitive (gtk_action_group_get_action
-				  (obj->priv->action_group_sensitive,
-				   "DirectoryPrevious"), FALSE);
-	gtk_action_set_sensitive (gtk_action_group_get_action
-				  (obj->priv->action_group_sensitive,
-				   "DirectoryNext"), FALSE);
+	action = gtk_action_group_get_action (obj->priv->action_group_sensitive,
+					      "DirectoryPrevious");
+	gtk_action_set_sensitive (action, FALSE);
+
+	action = gtk_action_group_get_action (obj->priv->action_group_sensitive,
+					      "DirectoryNext");
+	gtk_action_set_sensitive (action, FALSE);
 
 	toolbar = gtk_ui_manager_get_widget (manager, "/ToolBar");
 	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
@@ -982,6 +1043,20 @@ create_toolbar (GeditFileBrowserWidget * obj)
 	gtk_widget_show (toolbar);
 }
 
+static void 
+set_enable_delete (GeditFileBrowserWidget *obj,
+		   gboolean enable)
+{
+	GtkAction *action;
+	obj->priv->enable_delete = enable;
+	
+	action =
+	    gtk_action_group_get_action (obj->priv->action_group_selection,
+					 "FileDelete");
+	
+	g_object_set (action, "visible", enable, "sensitive", enable, NULL);
+}
+
 static gboolean
 filter_real (GeditFileBrowserStore * model, GtkTreeIter * iter,
 	     GeditFileBrowserWidget * obj)
@@ -1035,7 +1110,10 @@ add_bookmark_hash (GeditFileBrowserWidget * obj,
 	item->name = name;
 	item->icon = pixbuf;
 
-	g_hash_table_insert (obj->priv->bookmarks_hash, GUINT_TO_POINTER (gnome_vfs_uri_hash (guri)), item);
+	g_hash_table_insert (obj->priv->bookmarks_hash,
+			     GUINT_TO_POINTER (gnome_vfs_uri_hash (guri)),
+			     item);
+
 	gnome_vfs_uri_unref (guri);
 }
 
@@ -1069,7 +1147,6 @@ create_tree (GeditFileBrowserWidget * obj)
 	obj->priv->bookmarks_store = gedit_file_bookmarks_store_new ();
 	obj->priv->treeview =
 	    GEDIT_FILE_BROWSER_VIEW (gedit_file_browser_view_new ());
-	obj->priv->cursor = gdk_cursor_new (GDK_WATCH);
 
 	gedit_file_browser_store_set_filter_mode (obj->priv->file_store,
 						  GEDIT_FILE_BROWSER_STORE_FILTER_MODE_HIDE_HIDDEN
@@ -1121,7 +1198,7 @@ create_filter (GeditFileBrowserWidget * obj)
 	GtkWidget *vbox;
 	GtkWidget *entry;
 
-	expander = gtk_expander_new_with_mnemonic (_("_Advanced filtering"));
+	expander = gtk_expander_new_with_mnemonic (_("_Match Filename"));
 	gtk_widget_show (expander);
 	gtk_box_pack_start (GTK_BOX (obj), expander, FALSE, FALSE, 0);
 
@@ -1219,40 +1296,24 @@ update_sensitivity (GeditFileBrowserWidget * obj)
 			      (GTK_TREE_VIEW (obj->priv->treeview)), obj);
 }
 
-gboolean
-gedit_file_browser_widget_get_selected_directory (GeditFileBrowserWidget * obj, GtkTreeIter * iter)
+static gboolean
+gedit_file_browser_widget_get_first_selected (GeditFileBrowserWidget *obj,
+					      GtkTreeIter *iter)
 {
-	GtkTreeModel *model =
-	    gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
-	GtkTreeSelection *selection;
-	GtkTreeIter parent;
-	GtkTreeModel *selmodel;
-	guint flags;
-
-	if (!GEDIT_IS_FILE_BROWSER_STORE (model))
+	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (obj->priv->treeview));
+	GtkTreeModel *model;
+	GList *rows = gtk_tree_selection_get_selected_rows (selection, &model);
+	gboolean result;
+	
+	if (!rows)
 		return FALSE;
+	
+	result = gtk_tree_model_get_iter(model, iter, (GtkTreePath *)(rows->data));
 
-	selection =
-	    gtk_tree_view_get_selection (GTK_TREE_VIEW
-					 (obj->priv->treeview));
-
-	if (!gtk_tree_selection_get_selected (selection, &selmodel, iter)) {
-		gedit_file_browser_store_get_iter_virtual_root
-		    (GEDIT_FILE_BROWSER_STORE (model), iter);
-		selmodel = model;
-	}
-
-	gtk_tree_model_get (selmodel, iter,
-			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags,
-			    -1);
-
-	if (!FILE_IS_DIR (flags)) {
-		/* Get the parent, because the selection is a file */
-		gtk_tree_model_iter_parent (selmodel, &parent, iter);
-		*iter = parent;
-	}
-
-	return TRUE;
+	g_list_foreach (rows, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free (rows);
+	
+	return result;
 }
 
 static gboolean
@@ -1305,52 +1366,99 @@ filter_glob (GeditFileBrowserWidget * obj, GeditFileBrowserStore * store,
 	return result;
 }
 
-static gboolean
-delete_selected_file (GeditFileBrowserWidget * obj, gboolean trash)
+static void
+rename_selected_file (GeditFileBrowserWidget * obj)
 {
-	GtkTreeModel *model =
-	    gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
+	GtkTreeModel *model;
 	GtkTreeIter iter;
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
+
+	if (!GEDIT_IS_FILE_BROWSER_STORE (model))
+		return;
+
+	if (gedit_file_browser_widget_get_first_selected (obj, &iter))
+		gedit_file_browser_view_start_rename (obj->priv->treeview,
+						      &iter);
+}
+
+static GList *
+get_deletable_files (GeditFileBrowserWidget *obj) {
 	GtkTreeSelection *selection;
-	GtkTreeModel *selmodel;
+	GtkTreeModel *model;
+	GList *rows;
+	GList *row;
+	GList *paths = NULL;
 	guint flags;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
+
+	/* Get all selected files */
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (obj->priv->treeview));
+	rows = gtk_tree_selection_get_selected_rows (selection, &model);
+	
+	for (row = rows; row; row = row->next) {
+		path = (GtkTreePath *)(row->data);
+		
+		if (!gtk_tree_model_get_iter (model, &iter, path))
+			continue;
+		
+		gtk_tree_model_get (model, &iter, 
+				    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, 
+				    &flags, -1);
+
+		if (FILE_IS_DUMMY (flags))
+			continue;
+
+		paths = g_list_append (paths, gtk_tree_path_copy (path));
+	}
+	
+	g_list_foreach (rows, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free (rows);
+	
+	return paths;
+}
+
+static gboolean
+delete_selected_files (GeditFileBrowserWidget * obj, gboolean trash)
+{
+	GtkTreeModel *model;
 	gboolean confirm;
 	GeditFileBrowserStoreResult result;
+	GList *rows;
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
 
 	if (!GEDIT_IS_FILE_BROWSER_STORE (model))
 		return FALSE;
+		
+	rows = get_deletable_files (obj);
 
-	selection =
-	    gtk_tree_view_get_selection (GTK_TREE_VIEW
-					 (obj->priv->treeview));
-
-	if (!gtk_tree_selection_get_selected (selection, &selmodel, &iter))
-		return FALSE;
-
-	gtk_tree_model_get (model, &iter,
-			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags,
-			    -1);
-
-	if (FILE_IS_DUMMY (flags))
+	if (!rows)
 		return FALSE;
 
 	if (!trash) {
-		g_signal_emit (obj, signals[CONFIRM_DELETE], 0, model, &iter, &confirm);
+		g_signal_emit (obj, signals[CONFIRM_DELETE], 0, model, rows, &confirm);
 
 		if (!confirm)
 			return FALSE;
 	}
 
-	result = gedit_file_browser_store_delete (GEDIT_FILE_BROWSER_STORE (model),
-					          &iter, trash);
+	result = gedit_file_browser_store_delete_all (GEDIT_FILE_BROWSER_STORE (model),
+						      rows, trash);
 
 	if (result == GEDIT_FILE_BROWSER_STORE_RESULT_NO_TRASH) {
-		g_signal_emit (obj, signals[CONFIRM_NO_TRASH], 0, model, &iter, &confirm);
+		g_signal_emit (obj, signals[CONFIRM_NO_TRASH], 0, model, rows, &confirm);
 		
 		if (confirm)
-			result = gedit_file_browser_store_delete (GEDIT_FILE_BROWSER_STORE (model),
-					                          &iter, FALSE);
+			result = gedit_file_browser_store_delete_all (GEDIT_FILE_BROWSER_STORE (model),
+								      rows, FALSE);
 	}
+	
+	g_list_foreach (rows, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free (rows);
 
 	return result == GEDIT_FILE_BROWSER_STORE_RESULT_OK;
 }
@@ -1374,18 +1482,20 @@ get_topmost_uri (GnomeVFSURI const *uri)
 
 static GtkWidget *
 create_goto_menu_item (GeditFileBrowserWidget * obj, GList * item,
-		       GdkPixbuf * pixbuf)
+		       GdkPixbuf * icon)
 {
 	GtkWidget *result;
 	GtkWidget *image;
 	gchar *base;
 	gchar *unescape;
+	GdkPixbuf *pixbuf;
 	Location *loc;
 
 	loc = (Location *) (item->data);
 
-	if (!get_from_bookmark
-	    (obj, loc->virtual_root, &unescape, &pixbuf)) {
+	if (!get_from_bookmark (obj, loc->virtual_root,
+				&unescape, &pixbuf)) {
+
 		if (gedit_utils_uri_has_file_scheme (loc->virtual_root)) {
 			unescape =
 			    gnome_vfs_get_local_path_from_uri (loc->
@@ -1398,11 +1508,13 @@ create_goto_menu_item (GeditFileBrowserWidget * obj, GList * item,
 
 		unescape = gnome_vfs_unescape_string_for_display (base);
 		g_free (base);
-	} else {
-		unescape = g_strdup (unescape);
+
+		pixbuf = g_object_ref (icon);
 	}
 
 	image = gtk_image_new_from_pixbuf (pixbuf);
+	g_object_unref (pixbuf);
+
 	gtk_widget_show (image);
 
 	result = gtk_image_menu_item_new_with_label (unescape);
@@ -1812,74 +1924,126 @@ gedit_file_browser_widget_set_filter_pattern (GeditFileBrowserWidget * obj,
 	set_filter_pattern_real (obj, pattern, TRUE);
 }
 
+gboolean
+gedit_file_browser_widget_get_selected_directory (GeditFileBrowserWidget * obj, 
+						  GtkTreeIter * iter)
+{
+	GtkTreeModel *model =
+	    gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
+	GtkTreeIter parent;
+	guint flags;
+
+	if (!GEDIT_IS_FILE_BROWSER_STORE (model))
+		return FALSE;
+
+	if (!gedit_file_browser_widget_get_first_selected (obj, iter)) {
+		if (!gedit_file_browser_store_get_iter_virtual_root
+		    (GEDIT_FILE_BROWSER_STORE (model), iter))
+			return FALSE;
+	}
+
+	gtk_tree_model_get (model, iter,
+			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags,
+			    -1);
+
+	if (!FILE_IS_DIR (flags)) {
+		/* Get the parent, because the selection is a file */
+		gtk_tree_model_iter_parent (model, &parent, iter);
+		*iter = parent;
+	}
+
+	return TRUE;
+}
+
+guint
+gedit_file_browser_widget_get_num_selected_files_or_directories(GeditFileBrowserWidget *obj) {
+	GList *rows, *row;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	GeditFileBrowserStoreFlag flags;
+	guint result = 0;
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (obj->priv->treeview));
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
+
+	if (GEDIT_IS_FILE_BOOKMARKS_STORE (model))
+		return 0;
+	
+	rows = gtk_tree_selection_get_selected_rows (selection, &model);
+	
+	for (row = rows; row; row = row->next) {
+		path = (GtkTreePath *)(row->data);
+		
+		/* Get iter from path */
+		if (!gtk_tree_model_get_iter (model, &iter, path))
+			continue;
+
+		gtk_tree_model_get (model, &iter, GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags, -1);
+
+		if (!FILE_IS_DUMMY (flags))
+			++result;
+	}
+	
+	g_list_foreach (rows, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free (rows);
+	
+	return result;
+}
+
 /* Callbacks */
 static void
-on_row_activated (GtkTreeView * treeview, GtkTreePath * path,
-		  GtkTreeViewColumn * column, GeditFileBrowserWidget * obj)
+on_bookmark_activated (GeditFileBrowserView   *tree_view,
+		       GtkTreeIter            *iter,
+		       GeditFileBrowserWidget *obj)
 {
-	GtkTreeModel *model = gtk_tree_view_get_model (treeview);
+	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
+	gchar *uri;
+	gint flags;
+	
+	uri =
+	    gedit_file_bookmarks_store_get_uri
+	    (GEDIT_FILE_BOOKMARKS_STORE (model), iter);
+
+	gtk_tree_model_get (model, iter,
+			    GEDIT_FILE_BOOKMARKS_STORE_COLUMN_FLAGS,
+			    &flags, -1);
+
+	if (uri) {
+		if (flags & GEDIT_FILE_BOOKMARKS_STORE_IS_MOUNT) {
+			gedit_file_browser_widget_set_root (obj,
+							    uri,
+							    FALSE);
+		} else {
+			gedit_file_browser_widget_set_root (obj,
+							    uri,
+							    TRUE);
+		}
+	} else {
+		g_warning ("No uri!");
+	}
+
+	g_free (uri);
+}
+
+static void
+on_file_activated (GeditFileBrowserView   *tree_view, 
+		   GtkTreeIter            *iter,
+		   GeditFileBrowserWidget *obj)
+{
+	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
 	gchar *uri;
-	GtkTreeIter iter;
 	gint flags;
 
-	if (GEDIT_IS_FILE_BOOKMARKS_STORE (model)) {
-		gtk_tree_model_get_iter (model, &iter, path);
-		uri =
-		    gedit_file_bookmarks_store_get_uri
-		    (GEDIT_FILE_BOOKMARKS_STORE (model), &iter);
+	gtk_tree_model_get (model, iter,
+			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS,
+			    &flags,
+			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI,
+			    &uri, -1);
 
-		gtk_tree_model_get (model, &iter,
-				    GEDIT_FILE_BOOKMARKS_STORE_COLUMN_FLAGS,
-				    &flags, -1);
-
-		if (uri) {
-			if (flags & GEDIT_FILE_BOOKMARKS_STORE_IS_MOUNT) {
-				gedit_file_browser_widget_set_root (obj,
-								    uri,
-								    FALSE);
-			} else {
-				gedit_file_browser_widget_set_root (obj,
-								    uri,
-								    TRUE);
-			}
-		} else {
-			g_warning ("No uri!");
-		}
-
-		g_free (uri);
-	} else if (GEDIT_IS_FILE_BROWSER_STORE (model)) {
-		if (gtk_tree_model_get_iter (model, &iter, path)) {
-			gtk_tree_model_get (model, &iter,
-					    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS,
-					    &flags,
-					    GEDIT_FILE_BROWSER_STORE_COLUMN_URI,
-					    &uri, -1);
-
-			if (!FILE_IS_DIR (flags) && !FILE_IS_DUMMY (flags))
-				g_signal_emit (obj, signals[URI_ACTIVATED],
-					       0, uri);
-
-			g_free (uri);
-		}
-	}
-}
-
-static void
-on_begin_loading (GeditFileBrowserStore * model, GtkTreeIter * iter,
-		  GeditFileBrowserWidget * obj)
-{
-	if (GDK_IS_WINDOW (GTK_WIDGET (obj->priv->treeview)->window))
-		gdk_window_set_cursor (GTK_WIDGET (obj->priv->treeview)->window,
-				       obj->priv->cursor);
-}
-
-static void
-on_end_loading (GeditFileBrowserStore * model, GtkTreeIter * iter,
-		GeditFileBrowserWidget * obj)
-{
-	if (GDK_IS_WINDOW (GTK_WIDGET (obj->priv->treeview)->window))
-		gdk_window_set_cursor (GTK_WIDGET (obj->priv->treeview)->window,
-				       NULL);
+	g_signal_emit (obj, signals[URI_ACTIVATED], 0, uri);
+	g_free (uri);
 }
 
 static gboolean
@@ -2012,13 +2176,6 @@ on_model_set (GObject * gobject, GParamSpec * arg1,
 
 	clear_signals (obj);
 
-	add_signal (obj, gobject,
-		    g_signal_connect (gobject,
-				      "row-activated",
-				      G_CALLBACK
-				      (on_row_activated),
-				      obj));
-
 	if (GEDIT_IS_FILE_BOOKMARKS_STORE (model)) {
 		clear_next_locations (obj);
 
@@ -2042,15 +2199,17 @@ on_model_set (GObject * gobject, GParamSpec * arg1,
 		}
 
 		gtk_widget_set_sensitive (obj->priv->filter_expander, FALSE);
-	} else if (GEDIT_IS_FILE_BROWSER_STORE (model)) {
-		add_signal (obj, model,
-			    g_signal_connect (model, "begin-loading",
+
+
+		add_signal (obj, gobject,
+			    g_signal_connect (gobject, "bookmark-activated",
 					      G_CALLBACK
-					      (on_begin_loading), obj));
-		add_signal (obj, model,
-			    g_signal_connect (model, "end-loading",
-					      G_CALLBACK (on_end_loading),
-					      obj));
+					      (on_bookmark_activated), obj));
+	} else if (GEDIT_IS_FILE_BROWSER_STORE (model)) {
+		add_signal (obj, gobject,
+			    g_signal_connect (gobject, "file-activated",
+					      G_CALLBACK
+					      (on_file_activated), obj));
 		add_signal (obj, model,
 			    g_signal_connect (model,
 					      "notify::virtual-root",
@@ -2088,10 +2247,8 @@ on_combo_changed (GtkComboBox * combo, GeditFileBrowserWidget * obj)
 	guint id;
 	gchar *uri;
 
-	if (!gtk_combo_box_get_active_iter (combo, &iter)) {
-		g_warning ("Could not get iter in combo!");
+	if (!gtk_combo_box_get_active_iter (combo, &iter))
 		return;
-	}
 
 	gtk_tree_model_get (GTK_TREE_MODEL (obj->priv->combo_model), &iter,
 			    COLUMN_ID, &id, -1);
@@ -2181,12 +2338,13 @@ do_change_directory (GeditFileBrowserWidget * obj,
 	return FALSE;
 }
 
-	
 static gboolean
 on_treeview_key_press_event (GeditFileBrowserView * treeview,
 			     GdkEventKey * event,
 			     GeditFileBrowserWidget * obj)
 {
+	guint modifiers;
+
 	if (do_change_directory (obj, event))
 		return TRUE;
 
@@ -2194,13 +2352,25 @@ on_treeview_key_press_event (GeditFileBrowserView * treeview,
 	    (gtk_tree_view_get_model (GTK_TREE_VIEW (treeview))))
 		return FALSE;
 
+	modifiers = gtk_accelerator_get_default_mod_mask ();
+
 	if (event->keyval == GDK_Delete
-	    && !(event->state & GDK_CONTROL_MASK)) {
-		if (event->state & GDK_SHIFT_MASK) {
-			delete_selected_file (obj, FALSE);
-		} else {
-			delete_selected_file (obj, TRUE);
+	    || event->keyval == GDK_KP_Delete) {
+
+		if ((event->state & modifiers) == GDK_SHIFT_MASK) {
+			if (obj->priv->enable_delete) {
+				delete_selected_files (obj, FALSE);
+				return TRUE;
+			}
+		} else if ((event->state & modifiers) == 0) {
+			delete_selected_files (obj, TRUE);
+			return TRUE;
 		}
+	}
+
+	if ((event->keyval == GDK_F2)
+	    && (event->state & modifiers) == 0) {
+		rename_selected_file (obj);
 
 		return TRUE;
 	}
@@ -2212,29 +2382,18 @@ static void
 on_selection_changed (GtkTreeSelection * selection,
 		      GeditFileBrowserWidget * obj)
 {
-	gboolean has_selection = FALSE;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	guint flags;
+	guint has_selection = 0;
+	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
 
-	if (GEDIT_IS_FILE_BROWSER_STORE
-	    (gtk_tree_view_get_model
-	     (GTK_TREE_VIEW (obj->priv->treeview))))
-		has_selection =
-		    gtk_tree_selection_get_selected (selection, &model,
-						     &iter);
-
-	if (has_selection) {
-		gtk_tree_model_get (model, &iter,
-				    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS,
-				    &flags, -1);
-
-		if (FILE_IS_DUMMY (flags))
-			has_selection = FALSE;
-	}
+	if (GEDIT_IS_FILE_BROWSER_STORE(model))
+		has_selection = gedit_file_browser_widget_get_num_selected_files_or_directories (obj);
 
 	gtk_action_group_set_sensitive (obj->priv->action_group_selection,
-					has_selection);
+					has_selection > 0);
+	gtk_action_group_set_sensitive (obj->priv->action_group_single_selection,
+					has_selection == 1);
+	gtk_action_group_set_sensitive (obj->priv->action_group_single_most_selection,
+					has_selection <= 1);
 }
 
 static gboolean
@@ -2386,34 +2545,19 @@ on_action_file_new (GtkAction * action, GeditFileBrowserWidget * obj)
 static void
 on_action_file_rename (GtkAction * action, GeditFileBrowserWidget * obj)
 {
-	GtkTreeModel *model =
-	    gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
-	GtkTreeIter iter;
-	GtkTreeSelection *selection;
-	GtkTreeModel *selmodel;
-
-	if (!GEDIT_IS_FILE_BROWSER_STORE (model))
-		return;
-
-	selection =
-	    gtk_tree_view_get_selection (GTK_TREE_VIEW
-					 (obj->priv->treeview));
-
-	if (gtk_tree_selection_get_selected (selection, &selmodel, &iter))
-		gedit_file_browser_view_start_rename (obj->priv->treeview,
-						      &iter);
+	rename_selected_file (obj);
 }
 
 static void
 on_action_file_delete (GtkAction * action, GeditFileBrowserWidget * obj)
 {
-	delete_selected_file (obj, FALSE);
+	delete_selected_files (obj, FALSE);
 }
 
 static void
 on_action_file_move_to_trash (GtkAction * action, GeditFileBrowserWidget * obj)
 {
-	delete_selected_file (obj, TRUE);
+	delete_selected_files (obj, TRUE);
 }
 
 static void
@@ -2435,32 +2579,67 @@ on_action_directory_refresh (GtkAction * action,
 	}
 }
 
+static gboolean
+directory_open (GeditFileBrowserWidget *obj,
+		GtkTreeModel *model, 
+		GtkTreeIter *iter) {
+	gboolean result = FALSE;
+	GError *error = NULL;
+	gchar *uri = NULL;
+	GeditFileBrowserStoreFlag flags;
+	
+	gtk_tree_model_get (model, iter,
+			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags,
+			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, &uri, -1);
+
+	if (FILE_IS_DIR (flags)) {
+		result = TRUE;
+
+		if (!gnome_url_show (uri, &error)) {
+			g_signal_emit (obj, signals[ERROR], 0,
+				       GEDIT_FILE_BROWSER_ERROR_OPEN_DIRECTORY,
+				       error->message);
+			
+			g_error_free (error);
+			error = NULL;
+		}
+	}
+	
+	g_free (uri);
+	
+	return result;
+}
+
 static void
 on_action_directory_open (GtkAction * action, GeditFileBrowserWidget * obj)
 {
-	GtkTreeModel *model =
-	    gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
-	GtkTreeIter parent;
-	GError *error = NULL;
-	gchar *uri = NULL;
+	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (obj->priv->treeview));
+	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (obj->priv->treeview));	
+	GList *rows, *row;
+	gboolean directory_opened = FALSE;
+	GtkTreeIter iter;
+	GtkTreePath *path;
 
 	if (!GEDIT_IS_FILE_BROWSER_STORE (model))
 		return;
 
-	if (!gedit_file_browser_widget_get_selected_directory (obj, &parent))
-		return;
+	rows = gtk_tree_selection_get_selected_rows (selection, &model);
+	
+	for (row = rows; row; row = row->next) {
+		path = (GtkTreePath *)(row->data);
 
-
-	gtk_tree_model_get (model, &parent,
-			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, &uri, -1);
-
-	if (!gnome_url_show (uri, &error)) {
-		g_signal_emit (obj, signals[ERROR], 0,
-			       GEDIT_FILE_BROWSER_ERROR_OPEN_DIRECTORY,
-			       error->message);
+		if (gtk_tree_model_get_iter (model, &iter, path))
+			directory_opened |= directory_open (obj, model, &iter);
+		
+		gtk_tree_path_free (path);
+	}
+	
+	if (!directory_opened) {
+		if (gedit_file_browser_widget_get_selected_directory (obj, &iter))
+			directory_open (obj, model, &iter);
 	}
 
-	g_free (uri);
+	g_list_free (rows);
 }
 
 static void
